/***
 * Juliac
 * Copyright (C) 2008-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.opt.ultramerge;

import java.util.ArrayList;
import java.util.List;

import spoon.reflect.code.CtFieldAccess;
import spoon.reflect.declaration.CtClass;
import spoon.reflect.declaration.CtField;
import spoon.reflect.reference.CtFieldReference;
import spoon.reflect.visitor.Query;
import spoon.reflect.visitor.filter.AbstractFilter;

/**
 * A descriptor for a field of a component content class.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.3
 */
public class UMField<T> {

    /** The field. */
    private CtField<T> ctfield;
    
    /** The accesses to the field. */
    private List<CtFieldAccess<T>> fas;
    
    /**
     * @param ctfield  the field
     * @param scopes   the classes where accesses to the field must be sought
     */
    public UMField( final CtField<T> ctfield, List<CtClass<?>> scopes ) {
        
        this.ctfield = ctfield;
        fas = new ArrayList<CtFieldAccess<T>>();

        // Retrieve the accesses to ctfield contained in scopes
        for (CtClass<?> scope : scopes) {
            List<CtFieldAccess<T>> list =
                Query.getElements(
                    scope,
                    new AbstractFilter<CtFieldAccess<T>>(CtFieldAccess.class) {
                        public boolean matches( CtFieldAccess<T> fa ) {
                            CtFieldReference<T> fref = fa.getVariable();
                            CtField<T> f = fref.getDeclaration();
                            return (f==ctfield);
                        }
                    }
                );
            fas.addAll(list);
        }
    }

    public String getSimpleName() {
        return ctfield.getSimpleName();
    }
    
    public void setSimpleName( String name ) {
        ctfield.setSimpleName(name);
        for (CtFieldAccess<?> fa : fas) {
            CtFieldReference<?> fref = fa.getVariable();
            fref.setSimpleName(name);
        }
    }
    
    public void addCtFieldAccess( CtFieldAccess<T> fa ) {
        fas.add(fa);
    }
    
    public List<CtFieldAccess<T>> getCtFieldAccesses() {
        return fas;
    }
    
    public void removeCtFieldAccess( CtFieldAccess<T> fa ) {
        fas.remove(fa);
    }
    
    public CtField<?> getCtField() {
        return ctfield;
    }
    
    @Override
    public String toString() {
        return ctfield.getSimpleName();
    }
}
