/***
 * Juliac
 * Copyright (C) 2008-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.opt.ultramerge;

import java.util.ArrayList;
import java.util.List;

import spoon.reflect.code.CtExpression;
import spoon.reflect.code.CtFieldAccess;
import spoon.reflect.code.CtInvocation;
import spoon.reflect.declaration.CtClass;
import spoon.reflect.declaration.CtMethod;
import spoon.reflect.reference.CtExecutableReference;
import spoon.reflect.reference.CtFieldReference;
import spoon.reflect.reference.CtTypeReference;
import spoon.reflect.visitor.Query;
import spoon.reflect.visitor.filter.AbstractFilter;

/**
 * A descriptor for a method of a component content class.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.3
 */
public class UMMethod<T> {

    /** The method. */
    private CtMethod<T> ctmethod;
    
    /**
     * The invocations to the methods.
     */
    private List<CtInvocation<T>> invs;
    
    /**
     * @param ctmethod  the method
     * @param scopes
     * 		the classes where invocations to the method must be sought
     */
    public UMMethod( CtMethod<T> ctmethod, List<CtClass<?>> scopes ) {
        
        this.ctmethod = ctmethod;
        invs = new ArrayList<CtInvocation<T>>();

        // Retrieve the invocations to ctmethod contained in scopes
        final String er = ctmethod.getReference().toString();
        for (CtClass<?> scope : scopes) {
            List<CtInvocation<T>> list =
                Query.getElements(
                    scope,
                    new AbstractFilter<CtInvocation<T>>(CtInvocation.class) {
                        public boolean matches( CtInvocation<T> inv ) {
                            CtExecutableReference<?> eref = inv.getExecutable();
                            return er.equals(eref.toString());
                        }
                    }
                );
            invs.addAll(list);
        }
    }

    public String getSimpleName() {
        return ctmethod.getSimpleName();
    }
    
    public void setParent( CtClass<?> parent ) {
        ctmethod.setParent(parent);
        CtTypeReference<?> declaringType = parent.getReference();
        for (CtInvocation<?> inv : invs) {
            CtExecutableReference<?> eref = inv.getExecutable();
            eref.setDeclaringType(declaringType);
            CtExpression<?> expr = inv.getTarget();
            if( expr instanceof CtFieldAccess ) {
                CtFieldAccess<?> fa = (CtFieldAccess<?>) expr;
                fa.setTarget(null);
                fa.setType((CtTypeReference)declaringType);
                CtFieldReference<?> fref = fa.getVariable();
                fref.setDeclaringType(declaringType);
                fref.setSimpleName("this");
                fref.setType((CtTypeReference)declaringType);
            }
        }
    }
    
    public void setSimpleName( String name ) {
        ctmethod.setSimpleName(name);
        for (CtInvocation<?> inv : invs) {
            CtExecutableReference<?> eref = inv.getExecutable();
            eref.setSimpleName(name);
        }
    }
    
    public void addCtInvocation( CtInvocation<T> inv ) {
        invs.add(inv);
    }
    
    public List<CtInvocation<T>> getCtInvocations() {
        return invs;
    }
    
    public CtMethod<?> getCtMethod() {
        return ctmethod;
    }
    
    @Override
    public String toString() {
        return ctmethod.getSimpleName();
    }
}
